export class Main extends Indicator {

    constructor() {
        super();

        this.addInput("HISTORY_DEPTH", InputType.int, 16384);
        this.addInput("PriceType", InputType.PriceType, PriceType.Bid);

        this.buffers = {
            buf_st_max : this.addBuffer(),
            buf_mt_max : this.addBuffer(),
            buf_lt_max : this.addBuffer(),

            buf_st_min : this.addBuffer(),
            buf_mt_min : this.addBuffer(),
            buf_lt_min : this.addBuffer(),
        };

        this.arr = {
            arrMin : [],
            arrMax : [],
        }
    }

    onInit() {
        const {arrMin, arrMax} = this.arr,
            {HISTORY_DEPTH} = this.getInputs(),
            shortName = "AZZX - LARRY v2.0 ("+ HISTORY_DEPTH+ ")";

        this.buffers.buf_st_max
            .setShape(Shape.Arrow)
            .setColor(Color.Aqua)
            .setLabel("SHORT-TERM MAXIMUM")
            .setShortName(shortName)
            .setArrow(ArrowSymbols.CircleLarge)
        ;

        this.buffers.buf_mt_max
            .setShape(Shape.Arrow)
            .setColor(Color.Blue)
            .setLabel("MEDIUM-TERM MAXIMUM")
            .setShortName(shortName)
            .setArrow(ArrowSymbols.CircleLarge)
        ;

        this.buffers.buf_lt_max
            .setShape(Shape.Arrow)
            .setColor(Color.DeepPink)
            .setLabel("LONG-TERM MAXIMUM")
            .setShortName(shortName)
            .setArrow(ArrowSymbols.CircleLarge)
        ;

        this.buffers.buf_st_min
            .setShape(Shape.Arrow)
            .setColor(Color.Aqua)
            .setLabel("SHORT-TERM MINIMUM")
            .setShortName(shortName)
            .setArrow(ArrowSymbols.CircleLarge)
        ;

        this.buffers.buf_mt_min
            .setShape(Shape.Arrow)
            .setColor(Color.Blue)
            .setLabel("MEDIUM-TERM MINIMUM")
            .setShortName(shortName)
            .setArrow(ArrowSymbols.CircleLarge)
        ;

        this.buffers.buf_lt_min
            .setShape(Shape.Arrow)
            .setColor(Color.DeepPink)
            .setLabel("LONG-TERM MINIMUM")
            .setShortName(shortName)
            .setArrow(ArrowSymbols.CircleLarge)
        ;

        arrMin.length = HISTORY_DEPTH;
        arrMin.fill(0);

        arrMax.length = HISTORY_DEPTH;
        arrMax.fill(0);
    }

    async onUpdate() {
        const {HISTORY_DEPTH, PriceType} = this.getInputs(),
            {arrMin, arrMax} = this.arr,
            {buf_st_max, buf_mt_max, buf_lt_max, buf_st_min, buf_mt_min, buf_lt_min} = this.buffers,
            {High, Low} = await Bar.load([Bar.Mode.High, Bar.Mode.Low], PriceType),
            barSize = High.length;

        let i, l, r, time, lastBar = 0,
            historyDepth = Math.min(barSize, Math.max(0, HISTORY_DEPTH));


        time = await Bar.time(PriceType);

        if (lastBar === time[0]) {
            return 0;
        }

        lastBar = time[0];

        for (i = historyDepth - 2; i > 1; i--) {
            l = this.left_bar(historyDepth, i, High, Low);
            r = this.right_bar(i, High, Low);
            if ((l > 0) && (r > 0)) {
                if ((Low[l] > Low[i]) && (Low[r] > Low[i])) {
                    arrMin[i] = 1;
                }
            }
            if ((l > 0) && (r > 0)) {
                if ((High[l] < High[i]) && (High[r] < High[i])) {
                    arrMax[i] = 1;
                }
            }
        }
        for (i = historyDepth - 2; i > 1; i--) {
            l = this.left_st_min(historyDepth, i);
            r = this.right_st_min(i);
            if ((l > 0) && (r > 0)) {
                if ((Low[l] > Low[i]) && (Low[r] > Low[i])) {
                    arrMin[i] = 2;
                }
            }
            l = this.left_st_max(i);
            r = this.right_st_max(i);
            if ((l > 0) && (r > 0)) {
                if ((High[l] < High[i]) && (High[r] < High[i])) {
                    arrMax[i] = 2;
                }
            }
        }
        for (i = historyDepth - 2; i > 1; i--) {
            l = this.left_mt_min(historyDepth, i);
            r = this.right_mt_min(i);
            if ((l > 0) && (r > 0)) {
                if ((Low[l] > Low[i]) && (Low[r] > Low[i])) {
                    arrMin[i] = 3;
                }
            }
            l = this.left_mt_max(i);
            r = this.right_mt_max(i);
            if ((l > 0) && (r > 0)) {
                if ((High[l] < High[i]) && (High[r] < High[i])) {
                    arrMax[i] = 3;
                }
            }
        }

        buf_st_max.setSize(barSize).clear();
        buf_mt_max.setSize(barSize).clear();
        buf_lt_max.setSize(barSize).clear();
        buf_st_min.setSize(barSize).clear();
        buf_mt_min.setSize(barSize).clear();
        buf_lt_min.setSize(barSize).clear();

        for (i = historyDepth - 1; i > 0; i--) {
            switch (arrMin[i]) {
                case 1 :
                    buf_st_min.set(i, Low[i]);
                    break;
                case 2 :
                    buf_mt_min.set(i, Low[i]);
                    break;
                case 3 :
                    buf_lt_min.set(i, Low[i]);
                    break;
            }
            switch (arrMax[i]) {
                case 1 :
                    buf_st_max.set(i, High[i]);
                    break;
                case 2 :
                    buf_mt_max.set(i, High[i]);
                    break;
                case 3 :
                    buf_lt_max.set(i, High[i]);
                    break;
            }
        }
    }




    is_internal(bar, high, low) {
        return ((high[bar] < high[bar + 1]) && (low[bar] > low[bar + 1]));
    }

    left_bar(historyDepth, bar, high, low) {
        let i;
        for (i = bar + 1; i < historyDepth - 2; i++) {
            if (this.is_internal(i, high, low) === false) {
                return (i);
            }
        }
        return (-1);
    }

    right_bar(bar, high, low) {
        let i;
        for (i = bar - 1; i > 0; i--) {
            if (this.is_internal(i, high, low) === false) {
                return (i);
            }
        }
        return (-1);
    }

    left_st_min(historyDepth, bar) {
        let i;
        for (i = bar + 1; i < historyDepth - 2; i++) {
            if (this.arr.arrMin[i] > 0) {
                return (i);
            }
        }
        return (-1);
    }

    right_st_min(bar) {
        let i;
        for (i = bar - 1; i > 0; i--) {
            if (this.arr.arrMin[i] > 0) {
                return (i);
            }
        }
        return (-1);
    }

    left_st_max(historyDepth, bar) {
        let i;
        for (i = bar + 1; i < historyDepth - 2; i++) {
            if (this.arr.arrMax[i] > 0) {
                return (i);
            }
        }
        return (-1);
    }

    right_st_max(bar) {
        let i;
        for (i = bar - 1; i > 0; i--) {
            if (this.arr.arrMax[i] > 0) {
                return (i);
            }
        }
        return (-1);
    }

    left_mt_min(historyDepth, bar) {
        let i;
        for (i = bar + 1; i < historyDepth - 2; i++) {
            if (this.arr.arrMin[i] > 1) {
                return (i);
            }
        }
        return (-1);
    }

    right_mt_min(bar) {
        let i;
        for (i = bar - 1; i > 0; i--) {
            if (this.arr.arrMin[i] > 1) {
                return (i);
            }
        }
        return (-1);
    }

    left_mt_max(historyDepth, bar) {
        let i;
        for (i = bar + 1; i < historyDepth - 2; i++) {
            if (this.arr.arrMax[i] > 1) {
                return (i);
            }
        }
        return (-1);
    }

    right_mt_max(bar) {
        let i;
        for (i = bar - 1; i > 0; i--) {
            if (this.arr.arrMax[i] > 1) {
                return (i);
            }
        }
        return (-1);
    }
}
